--
-- $Id: Utils.hs,v 1.12 2006/05/31 15:20:19 aamine Exp $
--
-- Copyright (c) 2003-2006 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LPGL, see the file "COPYING".
--

module Utils
    (dirEntries, makePath, errorExit, packList) where

import PathUtils
import System.IO
import System.IO.Error
import System.Directory
import System.Exit

dirEntries :: FilePath -> IO [String]
dirEntries path = return . filter notDotFile =<< getDirectoryContents path
  where
    notDotFile ('.':_)  = False
    notDotFile _        = True

makePath :: FilePath -> IO ()
makePath = mkdirs . ancestors
  where
    mkdirs :: [String] -> IO ()
    mkdirs [] = return ()
    mkdirs (x:xs) =
        catch (mkdir_f x)
              (\ex -> if isDoesNotExistError ex then
                          mkdirs xs >>
                          mkdir_f x
                      else
                          ioError ex)

    mkdir_f path =
        catch (createDirectory path)
              (\ex -> if isAlreadyExistsError ex then
                          return ()
                      else
                          ioError ex)

errorExit :: String -> IO a
errorExit msg = do hPutStrLn stderr msg
                   exitWith (ExitFailure 1)

packList :: Int -> [String] -> String
packList max = unlines . map unwords . makeMatrix
  where
    makeMatrix :: [String] -> [[String]]
    makeMatrix []          = []
    makeMatrix list@(x:xs) = let (y,ys) = takeList list
                             in case y of
                                  [] -> [x] : makeMatrix xs
                                  _  -> y   : makeMatrix ys

    takeList :: [String] -> ([String], [String])
    takeList xs = let (a,b) = break ((> (max+1)) . fst) $ zip (lengthSum xs) xs
                  in (map snd a, map snd b)

    lengthSum :: [String] -> [Int]
    lengthSum = scanl1 (+) . map ((+ 1) . length)
